@@ -99,11 +99,20 @@ class PropertyType(Enum):
     INT_VALUE = "INT_VALUE"
     FLOAT_VALUE = "FLOAT_VALUE"
     DATETIME_VALUE = "DATETIME_VALUE"
+    DATE_VALUE = "DATE_VALUE"
+    TIME_VALUE = "TIME_VALUE"
     ATTACHMENT_VALUE = "ATTACHMENT_VALUE"
+    JSON_VALUE = "JSON_VALUE"
     LINK_VALUE = "LINK_VALUE"
-    SINGLE_SELECT_VALUE = 'SINGLE_SELECT_VALUE'
-    REFERENCE_VALUE = 'REFERENCE_VALUE'
+    SINGLE_SELECT_VALUE = "SINGLE_SELECT_VALUE"
+    MULTI_SELECT_VALUE = "MULTI_SELECT_VALUE"
+    REFERENCE_VALUE = "REFERENCE_VALUE"
     REFERENCES_VALUE = "REFERENCES_VALUE"
+    ACTIVITY_REFERENCES_VALUE = "ACTIVITY_REFERENCES_VALUE"
+    SCOPE_REFERENCES_VALUE = "SCOPE_REFERENCES_VALUE"
+    SERVICE_REFERENCES_VALUE = "SERVICE_REFERENCES_VALUE"
+    TEAM_REFERENCES_VALUE = "TEAM_REFERENCES_VALUE"
+    USER_REFERENCES_VALUE = "USER_REFERENCES_VALUE"
 
 
 class ActivityType(Enum):
@@ -113,35 +122,11 @@ class ActivityType(Enum):
 
     :cvar TASK: a normal task
     :cvar PROCESS: a subprocess (container) containing other tasks
-
-    For WIM version 1:
-
-    :cvar USERTASK: a normal usertask
-    :cvar SUBPROCESS: a subprocess (container) containing other tasks
-    :cvar SERVICETASK: a service taks (this concept is only availabe in RND KE-chain and will be deprecated)
     """
 
-    # WIM2:
     PROCESS = 'PROCESS'
     TASK = 'TASK'
 
-    # WIM1:
-    USERTASK = "UserTask"
-    SERVICETASK = "ServiceTask"  # RND code only
-    SUBPROCESS = "Subprocess"
-
-
-WIMCompatibleActivityTypes = {
-    # backwards pykechain script compatible with wim2
-    # from WIM1 to WIM2:
-    ActivityType.USERTASK: ActivityType.TASK,
-    ActivityType.SUBPROCESS: ActivityType.PROCESS,
-
-    # forwarde pykechain scripts made for wim2, compatible with wim1
-    # from WIM2 to WIM1:
-    ActivityType.PROCESS: ActivityType.SUBPROCESS,
-    ActivityType.TASK: ActivityType.USERTASK}
-
 
 class ActivityClassification(Enum):
     """The classification of Activities that are accepted by KE-chain.
@@ -467,6 +452,32 @@ class TeamRoles(Enum):
     OWNER = "OWNER"
 
 
+class ScopeRoles(Enum):
+    """
+    Roles that exist for a member of a scope.
+
+    :cvar MANAGER: owner of the scope, has full rights
+    :cvar LEADMEMBER: elevated member, has assignment rights
+    :cvar MEMBER: normal member, only has viewing rights
+    """
+
+    MANAGER = "manager"
+    LEADMEMBER = "leadmember"
+    MEMBER = "member"
+
+
+class ScopeMemberActions(Enum):
+    """
+    Actions to be performed on the members of a scope.
+
+    :cvar ADD: add a member to the scope
+    :cvar REMOVE: delete a member from the scope
+    """
+
+    ADD = 'add'
+    REMOVE = 'remove'
+
+
 class KechainEnv(Enum):
     """Environment variables that can be set for pykechain.
 
@@ -503,24 +514,25 @@ class SortTable(Enum):
     DESCENDING = 'DESC'
 
 
-class NavigationBarAlignment(Enum):
-    """The acceptable alignment options for a Navigation Bar Widget.
+class Alignment(Enum):
+    """The acceptable alignment options for attachment viewer and navigation bar widgets.
 
-    :cvar START: Buttons are aligned to the left (for KE-chain 2)
-    :cvar LEFT: Buttons are aligned to the left (for KE-chain 3)
-    :cvar CENTER: Buttons are aligned to the center
+    :cvar LEFT: Aligned to the left
+    :cvar CENTER: Aligned to the center
+    :cvar RIGHT: Aligned to the right
     """
 
-    CENTER = 'center'
-
-    # for KE-chain 2
-    START = 'start'
-
-    # for KE-chain 3
     LEFT = 'left'
+    CENTER = 'center'
     RIGHT = 'right'
 
 
+class NavigationBarAlignment(Alignment):
+    """The acceptable alignment options for a Navigation Bar Widget."""
+
+    pass
+
+
 class PaperSize(Enum):
     """The acceptable paper sizes options for a downloaded PDF.
 
@@ -623,6 +635,20 @@ class PropertyRepresentation(Enum):
     THOUSANDS_SEPARATOR = "thousandsSeparator"
 
 
+class OtherRepresentations(Enum):
+    """
+    Other representations used in KE-chain.
+
+    :cvar CUSTOM_ICON: different font-awesome icons
+    """
+
+    CUSTOM_ICON = 'customIcon'
+
+
+class _AllRepresentations(PropertyRepresentation, OtherRepresentations):
+    pass
+
+
 class ShowColumnTypes(Enum):
     """The columns that can be shown in a Property grid.
 
@@ -757,6 +783,7 @@ class CardWidgetLinkValue(Enum):
 
     EXTERNAL_LINK = "External link"
     TASK_LINK = "Task link"
+    TREE_VIEW = "Tree view"
     NO_LINK = "No link"
 
 
@@ -909,3 +936,63 @@ class WidgetTitleValue(Enum):
     DEFAULT = 'Default'
     NO_TITLE = 'No title'
     CUSTOM_TITLE = 'Custom title'
+
+
+class NotificationStatus(Enum):
+    """
+    Options to retrieve a Notification based on its status.
+
+    normal lifecycle:
+    - DRAFT, when a message is first saved to the backend and the status is still in draft.
+        next states: READY
+    - READY: when the message is ready for processing, it is complete and is to be processed
+        next states: PROCESSING
+    - PROCESSING: when the message is in the process of being send out
+        next states: COMPLETED, FAILED
+    - COMPLETED: when the message is successfully sent out
+        next states: ARCHIVED
+    - FAILED: when the message is not successfully sent out
+        next states: ARCHIVED
+    - ARCHIVED: when the message is archives and waiting for its deletion against a certain retention policy
+        next states: None
+
+    :cvar ARCHIVED: "archived" notifications
+    :cvar COMPLETED: "completed" notifications
+    :cvar DRAFT: "draft" notifications
+    :cvar FAILED: "failed" notifications
+    :cvar PROCESSING: "processing" notifications
+    :cvar READY: "ready" notifications
+    """
+
+    ARCHIVED = 'ARCHIVED'
+    COMPLETED = 'COMPLETED'
+    DRAFT = 'DRAFT'
+    FAILED = 'FAILED'
+    PROCESSING = 'PROCESSING'
+    READY = 'READY'
+
+
+class NotificationEvent(Enum):
+    """
+    Options to retrieve a Notification based on its event.
+
+    :cvar SHARE_ACTIVITY_LINK: notifications generated by sharing the link of an `Activity`
+    :cvar EXPORT_ACTIVITY_ASYNC: notifications generated by exporting an `Activity`
+    :cvar SHARE_ACTIVITY_PDF: notifications generated by sharing the pdf of an `Activity`
+    """
+
+    SHARE_ACTIVITY_LINK = 'SHARE_ACTIVITY_LINK'
+    EXPORT_ACTIVITY_ASYNC = 'EXPORT_ACTIVITY_ASYNC'
+    SHARE_ACTIVITY_PDF = 'SHARE_ACTIVITY_PDF'
+
+
+class NotificationChannels(Enum):
+    """
+    Options to retrieve a Notification based on its channel.
+
+    :cvar EMAIL: email notification
+    :cvar APP: app notification
+    """
+
+    EMAIL = 'EMAIL'
+    APP = 'APP'
