@@ -1,18 +1,21 @@
-from typing import Any, List, Dict, Optional, Text  # noqa: F401
+from typing import Any, List, Dict, Optional, Text, Union, Tuple, Iterable, TypeVar
 
 import requests
 from jsonschema import validate
-from six import text_type, iteritems
 
-from pykechain.enums import PropertyType, Category
+from pykechain.enums import Category
 from pykechain.exceptions import APIError, IllegalArgumentError
-from pykechain.models.property import Property
-from pykechain.models.representations.representation_base import BaseRepresentation
+from pykechain.models import Base, BaseInScope
+from pykechain.models.input_checks import check_text, check_type
+from pykechain.models.representations.component import RepresentationsComponent
+from pykechain.models.validators import PropertyValidator
 from pykechain.models.validators.validator_schemas import options_json_schema
 from pykechain.defaults import API_EXTRA_PARAMS
 
+T = TypeVar("T")
 
-class Property2(Property):
+
+class Property2(BaseInScope):
     """A virtual object representing a KE-chain property.
 
     .. versionadded: 3.0
@@ -32,8 +35,6 @@ class Property2(Property):
     :type model: str
     :ivar output: a boolean if the value is configured as an output (in an activity)
     :type output: bool
-    :ivar scope_id: the id of the Scope (not the `Scope` object)
-    :type scope_id: str
     :ivar part: The (parent) part in which this property is available
     :type part: :class:`Part2`
     :ivar value: the property value, can be set as well as property
@@ -46,21 +47,16 @@ class Property2(Property):
     :type is_invalid: bool
     """
 
-    use_bulk_update = False
+    _use_bulk_update = False
     _update_package = list()
 
     def __init__(self, json, **kwargs):
         """Construct a Property from a json object."""
-        super(Property2, self).__init__(json, **kwargs)
-
-        self._output = json.get('output')
-        self._value = json.get('value')
-        self._options = json.get('value_options')
-        self._part = None  # Part2 storage
-        self._model = None  # Model object storage
+        super().__init__(json, **kwargs)
 
+        self.output = json.get('output')  # type: bool
+        self.model_id = json.get('model_id')  # type: Optional[Text]
         self.part_id = json.get('part_id')
-        self.scope_id = json.get('scope_id')
         self.ref = json.get('ref')
         self.type = json.get('property_type')
         self.category = json.get('category')
@@ -68,16 +64,59 @@ class Property2(Property):
         self.unit = json.get('unit', None)
         self.order = json.get('order')
 
-        # set an empty internal validators variable
-        self._validators = []  # type: List[Any]
-        self._representations = []  # type: List[BaseRepresentation]
+        # Create protected variables
+        self._value = json.get('value')  # type: Any
+        self._options = json.get('value_options', {})  # type: Dict
+        self._part = None  # type: Optional['Part2']
+        self._model = None  # type: Optional['Property2']
+        self._validators = []  # type: List[PropertyValidator]
+        self._validation_results = []  # type: List
+        self._validation_reasons = []  # type: List
 
-        if self._options:
-            validate(self._options, options_json_schema)
-            if self._options.get('validators'):
-                self._parse_validators()
-            if self._options.get('representations'):
-                self._parse_representations()
+        self._representations_container = RepresentationsComponent(
+            self,
+            self._options.get('representations', {}),
+            self._save_representations,
+        )
+
+        validate(self._options, options_json_schema)
+
+        if self._options.get('validators'):
+            self._parse_validators()
+
+    def refresh(self, json: Optional[Dict] = None, url: Optional[Text] = None, extra_params: Optional = None) -> None:
+        """Refresh the object in place."""
+        super().refresh(json=json,
+                        url=self._client._build_url('property2', property_id=self.id),
+                        extra_params=API_EXTRA_PARAMS['property2'])
+
+    def has_value(self) -> bool:
+        """Predicate to indicate if the property has a value set.
+
+        This predicate determines if the property has a value set. It will not make a call to KE-chain API (in case
+        of reference properties). So it is a tiny fraction 'cheaper' in terms of processing time than checking the
+        `Property.value` itself.
+
+        It will return True if the property_type is a Boolean and set to a value of False.
+
+        :returns: True if the property has a value set, otherwise (also when value is None) returns False
+        :rtype: Bool
+        """
+        if isinstance(self._value, (float, int, bool)):
+            return True  # to prevent "bool(0.00) = False" or "False = False"
+        else:
+            return bool(self._value)
+
+    @property
+    def use_bulk_update(self):
+        """Set or get the toggle to asynchronously update property values."""
+        # set the class attribute to make this value a singleton
+        return self.__class__._use_bulk_update
+
+    @use_bulk_update.setter
+    def use_bulk_update(self, value):
+        assert isinstance(value, bool), "`use_bulk_update` must be set to a boolean, not {}".format(type(value))
+        self.__class__._use_bulk_update = value
 
     @property
     def value(self) -> Any:
@@ -92,11 +131,7 @@ class Property2(Property):
     @value.setter
     def value(self, value: Any) -> None:
         if self.use_bulk_update:
-            self.__class__._update_package.append(dict(
-                id=self.id,
-                value=value,
-            ))
-            self._value = value
+            self._pend_value(value)
         else:
             self._put_value(value)
 
@@ -116,34 +151,43 @@ class Property2(Property):
             cls._update_package = list()
         cls.use_bulk_update = use_bulk_update
 
+    def _pend_value(self, value):
+        """Store the value to be send at a later point in time using `update_values`."""
+        value = self.serialize_value(value)
+
+        self.__class__._update_package.append(dict(
+            id=self.id,
+            value=value,
+        ))
+        self._value = value
+        return self._value
+
     def _put_value(self, value):
+        """Send the value to KE-chain."""
+        value = self.serialize_value(value)
+
         url = self._client._build_url('property2', property_id=self.id)
 
         response = self._client._request('PUT', url, params=API_EXTRA_PARAMS['property2'], json={'value': value})
 
         if response.status_code != requests.codes.ok:  # pragma: no cover
-            raise APIError("Could not update property value: '{}'".format(response.content))
+            raise APIError("Could not update Property {}".format(self), response=response)
 
         self.refresh(json=response.json()['results'][0])
         return self._value
 
-    @property
-    def model_id(self):
-        # type: () -> (type(None), Part2)  # noqa: F821
-        """Model id of the Property.
-
-        Returns None if the property is a model of its own. It will not return the model object, only the uuid.
-
-        It will return None if the `model_id_name` field is not found in the response of the server.
+    def serialize_value(self, value: [T]) -> T:
         """
-        if self.category == Category.MODEL:
-            return None
-        else:
-            return self._json_data.get('model_id')
+        Serialize the value to be set on the property.
+
+        :param value: non-serialized value
+        :type value: Any
+        :return: serialized value
+        """
+        return value.id if isinstance(value, Base) else value
 
     @property
-    def part(self):
-        # type: () -> Part2  # noqa: F821
+    def part(self) -> 'Part2':
         """
         Retrieve the part that holds this Property.
 
@@ -154,65 +198,147 @@ class Property2(Property):
             self._part = self._client.part(pk=self.part_id, category=self.category)
         return self._part
 
+    def model(self) -> 'AnyProperty':
+        """
+        Model object of the property if the property is an instance otherwise itself.
+
+        Will cache the model object in order to not generate too many API calls. Otherwise will make an API call
+        to the backend to retrieve its model object.
+
+        :return: `Property` model object if the current `Property` is an instance.
+        :rtype: :class:`pykechain.models.AnyProperty`
+        """
+        if self.category == Category.MODEL:
+            return self
+        elif self._model is None:
+            self._model = self._client.property(pk=self.model_id, category=Category.MODEL)
+        return self._model
+
     @property
-    def representations(self):
-        # type: () -> List[BaseRepresentation]
-        """
-        Provide list of representation objects.
+    def validators(self):
+        """Provide list of Validator objects.
 
-        :return: list of Representations
+        :returns: list of :class:`PropertyValidator` objects
+        :rtype: list(PropertyValidator)
         """
-        return self._representations
+        return self._validators
 
-    @representations.setter
-    def representations(self, representations):
-        # type: (List[BaseRepresentation]) -> None
+    @validators.setter
+    def validators(self, validators: Iterable[PropertyValidator]) -> None:
         if self.category != Category.MODEL:
-            raise IllegalArgumentError("To update the list of representations, it can only work on "
+            raise IllegalArgumentError("To update the list of validators, it can only work on "
                                        "`Property` of category 'MODEL'")
 
-        if not isinstance(representations, (tuple, list)):
-            raise IllegalArgumentError('Should be a list or tuple with Representation objects, '
-                                       'got {}'.format(type(representations)))
-        for representation in representations:
-            if not isinstance(representation, BaseRepresentation):
-                raise IllegalArgumentError(
-                    "Representation '{}' should be a Representation object".format(representation))
-            representation.validate_json()
+        if not isinstance(validators, (tuple, list)) or not all(isinstance(v, PropertyValidator) for v in validators):
+            raise IllegalArgumentError('Should be a list or tuple with PropertyValidator objects, '
+                                       'got {}'.format(type(validators)))
+        for validator in validators:
+            validator.validate_json()
 
-        # set the internal representation list
-        self._representations = list(set(representations))
+        # set the internal validators list
+        self._validators = list(set(validators))
 
         # dump to _json options
-        self._dump_representations()
+        self._dump_validators()
 
         # update the options to KE-chain backend
         self.edit(options=self._options)
 
-    def _parse_representations(self):
-        """Parse the representations in the options to representations."""
-        self._representations = []
-        representations_json = self._options.get('representations')
-        for representation_json in representations_json:
-            self._representations.append(BaseRepresentation.parse(prop=self, json=representation_json))
+    def _parse_validators(self):
+        """Parse the validator in the options to validators."""
+        self._validators = []
+        validators_json = self._options.get('validators')
+        for validator_json in validators_json:
+            self._validators.append(PropertyValidator.parse(json=validator_json))
 
-    def _dump_representations(self):
-        """Dump the representations as json inside the _options dictionary with the key `representations`."""
-        if hasattr(self, '_representations'):
-            representations_json = []
-            for representation in self._representations:
-                if isinstance(representation, BaseRepresentation):
-                    representations_json.append(representation.as_json())
-                else:
-                    raise APIError("representation is not a BaseRepresentation: '{}'".format(representation))
-            if self._options.get('representations', list()) == representations_json:
-                # no change
-                pass
+    def _dump_validators(self):
+        """Dump the validators as json inside the _options dictionary with the key `validators`."""
+        validators_json = []
+        for validator in self._validators:
+            if isinstance(validator, PropertyValidator):
+                validators_json.append(validator.as_json())
             else:
-                new_options = self._options.copy()  # make a copy
-                new_options.update({'representations': representations_json})
-                validate(new_options, options_json_schema)
-                self._options = new_options
+                raise APIError("validator is not a PropertyValidator: '{}'".format(validator))
+        if self._options.get('validators', list()) == validators_json:
+            # no change
+            pass
+        else:
+            new_options = self._options.copy()  # make a copy
+            new_options.update({'validators': validators_json})
+            validate(new_options, options_json_schema)
+            self._options = new_options
+
+    @property
+    def is_valid(self) -> Optional[bool]:
+        """Determine if the value in the property is valid.
+
+        If the value of the property is validated as 'valid', than returns a True, otherwise a False.
+        When no validators are configured, returns a None. It checks against all configured validators
+        and returns a single boolean outcome.
+
+        :returns: True when the `value` is valid
+        :rtype: bool or None
+        """
+        if not self._validators:
+            return None
+        else:
+            self.validate(reason=False)
+            if all([vr is None for vr in self._validation_results]):
+                return None
+            else:
+                return all(self._validation_results)
+
+    @property
+    def is_invalid(self) -> Optional[bool]:
+        """Determine if the value in the property is invalid.
+
+        If the value of the property is validated as 'invalid', than returns a True, otherwise a False.
+        When no validators are configured, returns a None. It checks against all configured validators
+        and returns a single boolean outcome.
+
+        :returns: True when the `value` is invalid
+        :rtype: bool
+        """
+        return not self.is_valid if self.is_valid is not None else None
+
+    def validate(self, reason: bool = True) -> List[Union[bool, Tuple]]:
+        """Return the validation results and include an (optional) reason.
+
+        If reason keyword is true, the validation is returned for each validation
+        the [(<result: bool>, <reason:str>), ...]. If reason is False, only a single list of validation results
+        for each configured validator is returned.
+
+        :param reason: (optional) switch to indicate if the reason of the validation should be provided
+        :type reason: bool
+        :return: list of validation results [bool, bool, ...] or
+                 a list of validation results, reasons [(bool, str), ...]
+        :rtype: list(bool) or list((bool, str))
+        :raises Exception: for incorrect validators or incompatible values
+        """
+        self._validation_results = [validator.is_valid(self._value) for validator in self._validators]
+        self._validation_reasons = [validator.get_reason() for validator in self._validators]
+
+        if reason:
+            return list(zip(self._validation_results, self._validation_reasons))
+        else:
+            return self._validation_results
+
+    @property
+    def representations(self):
+        """Get and set the property representations."""
+        return self._representations_container.get_representations()
+
+    @representations.setter
+    def representations(self, value):
+        if self.category != Category.MODEL:
+            raise IllegalArgumentError("To update the list of representations, it can only work on a "
+                                       "`Property` of category '{}'".format(Category.MODEL))
+
+        self._representations_container.set_representations(value)
+
+    def _save_representations(self, representation_options):
+        self._options.update({'representations': representation_options})
+        self.edit(options=self._options)
 
     @classmethod
     def create(cls, json: dict, **kwargs) -> 'AnyProperty':
@@ -228,48 +354,22 @@ class Property2(Property):
         """
         property_type = json.get('property_type')
 
-        # changed for PIM2
-        if property_type == PropertyType.ATTACHMENT_VALUE:
-            from .property2_attachment import AttachmentProperty2
-            return AttachmentProperty2(json, **kwargs)
-        elif property_type == PropertyType.SINGLE_SELECT_VALUE:
-            from .property2_selectlist import SelectListProperty2
-            return SelectListProperty2(json, **kwargs)
-        elif property_type == PropertyType.REFERENCES_VALUE:
-            from .property2_multi_reference import MultiReferenceProperty2
-            return MultiReferenceProperty2(json, **kwargs)
-        elif property_type == PropertyType.DATETIME_VALUE:
-            from .property2_datetime import DatetimeProperty2
-            return DatetimeProperty2(json, **kwargs)
-        else:
-            return Property2(json, **kwargs)
+        from pykechain.models import property_type_to_class_map
 
-    def refresh(self, json=None, url=None, extra_params=None):
-        # type: (Optional[Dict], Optional[Text], Optional) -> ()
-        """Refresh the object in place."""
-        super(Property2, self).refresh(json=json,
-                                       url=self._client._build_url('property2', property_id=self.id),
-                                       extra_params=API_EXTRA_PARAMS['property2'])
+        # Get specific Property subclass, defaulting to Property2 itself
+        property_class = property_type_to_class_map.get(property_type, Property2)
 
-    def has_value(self) -> bool:
-        """Predicate to indicate if the property has a value set.
+        # Call constructor and return new object
+        return property_class(json, **kwargs)
 
-        This predicate determines if the property has a value set. It will not make a call to KE-chain API (in case
-        of reference properties). So it is a tiny fraction 'cheaper' in terms of processing time than checking the
-        `Property.value` itself.
-
-        It will return True if the property_type is a Boolean and set to a value of False.
-
-        :returns: True if the property has a value set, otherwise (also when value is None) returns False
-        :rtype: Bool
-        """
-        if isinstance(self._value, (float, int, bool)):
-            return True  # to prevent "bool(0.00) = False" or "False = False"
-        else:
-            return bool(self._value)
-
-    def edit(self, name=None, description=None, unit=None, options=None, **kwargs):
-        # type: (Optional[Text], Optional[Text], Optional[Text], Optional[Dict], **Any) -> ()
+    def edit(
+            self,
+            name: Optional[Text] = None,
+            description: Optional[Text] = None,
+            unit: Optional[Text] = None,
+            options: Optional[Dict] = None,
+            **kwargs
+    ) -> None:
         """Edit the details of a property (model).
 
         :param name: (optional) new name of the property to edit
@@ -305,32 +405,22 @@ class Property2(Property):
         >>> wheel_property_reference.edit(options=options)
 
         """
-        update_dict = {'id': self.id}
-        if name is not None:
-            if not isinstance(name, (str, text_type)):
-                raise IllegalArgumentError(
-                    "name should be provided as a string, was provided as '{}'".format(type(name)))
-            update_dict.update({'name': name})
-            self.name = name
-        if description is not None:
-            if not isinstance(description, (str, text_type)):
-                raise IllegalArgumentError("description should be provided as a string, was provided as '{}'".
-                                           format(type(description)))
-            update_dict.update({'description': description})
-        if unit is not None:
-            if not isinstance(unit, (str, text_type)):
-                raise IllegalArgumentError("unit should be provided as a string, was provided as '{}'".
-                                           format(type(unit)))
-            update_dict.update({'unit': unit})
-        if options is not None:
-            if not isinstance(options, dict):
-                raise IllegalArgumentError("options should be provided as a dict, was provided as '{}'".
-                                           format(type(options)))
-            update_dict.update({'value_options': options})
-        if kwargs is not None:
-            # process the other kwargs in py27 style.
-            for key, value in iteritems(kwargs):
-                update_dict[key] = value
+        update_dict = {
+            'id': self.id,
+            'name': check_text(name, 'name') or self.name,
+            'description': check_text(description, 'description') or self.description,
+        }
+
+        options = check_type(options, dict, 'options')
+        if options:
+            update_dict['value_options'] = options
+
+        unit = check_text(unit, 'unit')
+        if unit:
+            update_dict['unit'] = unit
+
+        if kwargs:  # pragma: no cover
+            update_dict.update(kwargs)
 
         response = self._client._request('PUT',
                                          self._client._build_url('property2', property_id=self.id),
@@ -338,12 +428,11 @@ class Property2(Property):
                                          json=update_dict)
 
         if response.status_code != requests.codes.ok:  # pragma: no cover
-            raise APIError("Could not update Property ({})".format(response))
+            raise APIError("Could not update Property {}".format(self), response=response)
 
         self.refresh(json=response.json()['results'][0])
 
-    def delete(self):
-        # type: () -> ()
+    def delete(self) -> None:
         """Delete this property.
 
         :return: None
@@ -352,10 +441,9 @@ class Property2(Property):
         response = self._client._request('DELETE', self._client._build_url('property2', property_id=self.id))
 
         if response.status_code != requests.codes.no_content:  # pragma: no cover
-            raise APIError("Could not delete property: {} with id {}".format(self.name, self.id))
+            raise APIError("Could not delete Property {}".format(self), response=response)
 
-    def copy(self, target_part, name=None):
-        # type: (Part2, Optional[Text]) -> Property2
+    def copy(self, target_part: 'Part2', name: Optional[Text] = None) -> 'Property2':
         """Copy a property model or instance.
 
         :param target_part: `Part` object under which the desired `Property` is copied
@@ -373,10 +461,9 @@ class Property2(Property):
 
         """
         from pykechain.models import Part2
-        if not isinstance(target_part, Part2):
-            raise IllegalArgumentError("`target_part` needs to be a part, got '{}'".format(type(target_part)))
-        if not name:
-            name = self.name
+        check_type(target_part, Part2, 'target_part')
+
+        name = check_text(name, 'name') or self.name
         if self.category == Category.MODEL and target_part.category == Category.MODEL:
             # Cannot move a `Property` model under a `Part` instance or vice versa
             copied_property_model = target_part.add_property(name=name,
@@ -405,8 +492,7 @@ class Property2(Property):
             raise IllegalArgumentError('property "{}" and target part "{}" must have the same category'.
                                        format(self.name, target_part.name))
 
-    def move(self, target_part, name=None):
-        # type: (Part2, Optional[Text]) -> Property2  # noqa: F821
+    def move(self, target_part: 'Part2', name: Optional[Text] = None) -> 'Property2':
         """Move a property model or instance.
 
         :param target_part: `Part` object under which the desired `Property` is moved
