@@ -8,10 +8,10 @@ import pytz
 import requests
 
 from pykechain.enums import ActivityType, ActivityStatus, ActivityClassification, Category, \
-    activity_root_name_by_classification, ActivityRootNames
+    activity_root_name_by_classification, ActivityRootNames, PaperSize, PaperOrientation, NotificationEvent
 from pykechain.exceptions import NotFoundError, MultipleFoundError, IllegalArgumentError, APIError
 from pykechain.models import Activity2
-from pykechain.utils import temp_chdir
+from pykechain.utils import temp_chdir, slugify_ref
 from tests.classes import TestBetamax
 from tests.utils import TEST_FLAG_IS_WIM2
 
@@ -75,6 +75,20 @@ class TestActivityConstruction(TestBetamax):
         self.assertEqual(ActivityType.TASK, self.task.activity_type)
         self.assertEqual(ActivityClassification.WORKFLOW, self.task.classification)
 
+    def test_create_below_parent(self):
+        self.process.children()  # populate `_cached_children`.
+        self.assertIsNotNone(self.process._cached_children, 'Cached children should be an (empty) list.')
+
+        new_task = self.process.create(
+            name='__Testing task',
+            activity_type=ActivityType.TASK,
+        )
+
+        current_children = self.process.children()
+
+        self.assertTrue(current_children)
+        self.assertIn(new_task, current_children, msg='New child task should be among the children.')
+
     def test_create_with_classification(self):
 
         for classification in ActivityClassification.values():
@@ -165,8 +179,9 @@ class TestActivities(TestBetamax):
 
         obj = self.project.activity('Specify wheel diameter')
         for attribute in attributes:
-            self.assertTrue(hasattr(obj, attribute),
-                            "Could not find '{}' in the object: '{}'".format(attribute, obj.__dict__.keys()))
+            with self.subTest(msg=attribute):
+                self.assertTrue(hasattr(obj, attribute),
+                                "Could not find '{}' in the object: '{}'".format(attribute, obj.__dict__.keys()))
 
     def test_retrieve_unknown_activity(self):
         with self.assertRaises(NotFoundError):
@@ -208,8 +223,8 @@ class TestActivities(TestBetamax):
     def test_edit_activity_naive_dates(self):
         specify_wd = self.project.activity('Specify wheel diameter')
 
-        old_start, old_due = self._convert_timestamp(specify_wd._json_data.get('start_date')), \
-                             self._convert_timestamp(specify_wd._json_data.get('due_date'))
+        old_start, old_due = specify_wd.start_date, specify_wd.due_date
+
         start_time = datetime(2000, 1, 1, 0, 0, 0)
         due_time = datetime(2019, 12, 31, 0, 0, 0)
 
@@ -237,8 +252,7 @@ class TestActivities(TestBetamax):
         specify_wd = self.project.activity('Specify wheel diameter')
 
         # save old values
-        old_start, old_due = self._convert_timestamp(specify_wd._json_data.get('start_date')), \
-                             self._convert_timestamp(specify_wd._json_data.get('due_date'))
+        old_start, old_due = specify_wd.start_date, specify_wd.due_date
 
         startdate = datetime.now(pytz.utc)
         duedate = datetime(2019, 1, 1, 0, 0, 0, tzinfo=pytz.timezone('Europe/Amsterdam'))
@@ -268,16 +282,6 @@ class TestActivities(TestBetamax):
         # Return the status to how it used to be
         specify_wd.edit(status=original_status)
 
-    def _convert_timestamp(self, value):
-        try:
-            r = datetime.strptime(value, ISOFORMAT)
-        except ValueError:
-            r = datetime.strptime(value, ISOFORMAT_HIGHPRECISION)
-        if isinstance(r, datetime):
-            return r
-        else:
-            raise ValueError
-
     # 1.7.2
     def test_datetime_with_naive_duedate_only_fails(self):
         """reference to #121 - thanks to @joost.schut"""
@@ -285,8 +289,8 @@ class TestActivities(TestBetamax):
         specify_wd = self.project.activity('Specify wheel diameter')
 
         # save old values
-        old_start, old_due = self._convert_timestamp(specify_wd._json_data.get('start_date')), \
-                             self._convert_timestamp(specify_wd._json_data.get('due_date'))
+        old_start, old_due = specify_wd.start_date, specify_wd.due_date
+
         naive_duedate = datetime(2017, 6, 5, 5, 0, 0)
         with warnings.catch_warnings(record=False) as w:
             warnings.simplefilter("ignore")
@@ -305,8 +309,7 @@ class TestActivities(TestBetamax):
         # setup
         specify_wd = self.project.activity('Specify wheel diameter')
         # save old values
-        old_start, old_due = self._convert_timestamp(specify_wd._json_data.get('start_date')), \
-                             self._convert_timestamp(specify_wd._json_data.get('due_date'))
+        old_start, old_due = specify_wd.start_date, specify_wd.due_date
 
         tz = pytz.timezone('Europe/Amsterdam')
         tzaware_due = tz.localize(datetime(2017, 7, 1))
@@ -381,7 +384,7 @@ class TestActivities(TestBetamax):
         all_tasks = workflow_root.all_children()
 
         self.assertIsInstance(all_tasks, list)
-        self.assertEqual(12, len(all_tasks), msg='Number of tasks has changed, expected 12.')
+        self.assertEqual(11, len(all_tasks), msg='Number of tasks has changed, expected 12.')
 
     def test_retrieve_activity_by_id(self):
         task = self.project.activity(name='Subprocess')  # type: Activity2
@@ -397,6 +400,11 @@ class TestActivities(TestBetamax):
         self.assertIn(task.id, [sibling.id for sibling in siblings])
         self.assertTrue(len(siblings) >= 1)
 
+    def test_retrieve_siblings_of_root(self):
+        task = self.project.activity(name=ActivityRootNames.WORKFLOW_ROOT)
+        with self.assertRaises(NotFoundError):
+            task.siblings()
+
     # in 1.12
 
     def test_retrieve_siblings_of_a_task_in_a_subprocess_with_arguments(self):
@@ -416,12 +424,23 @@ class TestActivities(TestBetamax):
 
     # in 1.13
     def test_create_activity_with_incorrect_activity_class_fails(self):
-        with self.assertRaisesRegex(IllegalArgumentError, 'Please provide accepted activity_type'):
+        with self.assertRaisesRegex(IllegalArgumentError, 'must be an option from enum'):
             self.project.create_activity(name='New', activity_type='DEFUNCTActivity')
 
 
 @skipIf(not TEST_FLAG_IS_WIM2, reason="This tests is designed for WIM version 2, expected to fail on older WIM")
 class TestActivity2SpecificTests(TestBetamax):
+
+    def setUp(self):
+        super().setUp()
+        self.root = self.project.activity(ActivityRootNames.WORKFLOW_ROOT)
+        self.task = self.root.create(name='test task', activity_type=ActivityType.TASK)
+
+    def tearDown(self):
+        if self.task:
+            self.task.delete()
+        super().tearDown()
+
     # 2.0 new activity
     # noinspection PyTypeChecker
     def test_edit_activity2_assignee(self):
@@ -455,6 +474,11 @@ class TestActivity2SpecificTests(TestBetamax):
         subprocess = task.parent()  # type Activity
         self.assertEqual(subprocess.activity_type, ActivityType.PROCESS)
 
+    def test_activity2_retrieve_parent_of_root(self):
+        task = self.project.activity(name=ActivityRootNames.WORKFLOW_ROOT)
+        with self.assertRaises(NotFoundError):
+            task.parent()
+
     def test_activity2_retrieve_parent_of_a_toplevel_task_returns_workflow_root_id(self):
         task = self.project.activity('Specify wheel diameter')
         parent = task.parent()
@@ -485,11 +509,14 @@ class TestActivity2SpecificTests(TestBetamax):
 
         self.assertTrue(specify_wd.is_rootlevel())
 
+        root_itself = self.project.activity(ActivityRootNames.WORKFLOW_ROOT)
+
+        self.assertFalse(root_itself.is_rootlevel())
+
     def test_subtask_activity2_is_not_rootlevel(self):
         subprocess_subtask = self.project.activity('SubTask')
 
         self.assertFalse(subprocess_subtask.is_rootlevel())
-        # self.assertTrue(subprocess_subtask.subprocess())
 
     def test_activity2_is_task(self):
         specify_wd = self.project.activity('Specify wheel diameter')
@@ -504,11 +531,8 @@ class TestActivity2SpecificTests(TestBetamax):
         self.assertFalse(subprocess.is_task())
 
     def test_activity2_assignees_list(self):
-        activity_name = 'test task'
-        activity = self.project.activity(name=activity_name)  # type: Activity2
-
-        list_of_assignees_in_data = activity._json_data.get('assignees_ids')
-        assignees_list = activity.assignees
+        list_of_assignees_in_data = self.task._json_data.get('assignees_ids')
+        assignees_list = self.task.assignees
 
         self.assertSetEqual(set(list_of_assignees_in_data), set([u.id for u in assignees_list]))
 
@@ -519,63 +543,47 @@ class TestActivity2SpecificTests(TestBetamax):
         list_of_assignees_in_data = activity._json_data.get('assignees_ids')
         assignees_list = activity.assignees
 
-        self.assertListEqual(list(), activity.assignees, "Task has no assingees and should return Empty list")
+        self.assertListEqual(list(), activity.assignees, "Task has no assignees and should return Empty list")
 
     def test_activity2_move(self):
         # setUp
-        activity_name = 'test task'
-        activity_to_be_moved = self.project.activity(name=activity_name)
-        original_parent = activity_to_be_moved.parent()
+        activity_to_be_moved = self.task
 
         new_parent_name = 'Subprocess'
         new_parent = self.project.activity(name=new_parent_name)
 
         activity_to_be_moved.move(parent=new_parent)
 
-        activity_to_be_moved.refresh()
-
         # testing
-        self.assertTrue(activity_to_be_moved.parent().id == new_parent.id)
-
-        # tearDown
-        activity_to_be_moved.move(parent=original_parent.id)
-        activity_to_be_moved.refresh()
-        self.assertTrue(activity_to_be_moved.parent().id == original_parent.id)
+        self.assertEqual(new_parent, activity_to_be_moved.parent())
 
     def test_activity2_move_under_task_parent(self):
         # setUp
-        activity_name = 'test task'
-        activity_to_be_moved = self.project.activity(name=activity_name)
-
         new_parent_name = 'Specify wheel diameter'
         new_parent = self.project.activity(name=new_parent_name)
 
         # testing
         with self.assertRaises(IllegalArgumentError):
-            activity_to_be_moved.move(parent=new_parent)
+            self.task.move(parent=new_parent)
 
     def test_activity2_move_under_part_object(self):
         # setUp
-        activity_name = 'test task'
-        activity_to_be_moved = self.project.activity(name=activity_name)
-
         new_parent_name = 'Bike'
         new_parent = self.project.part(name=new_parent_name)
 
         # testing
         with self.assertRaises(IllegalArgumentError):
-            activity_to_be_moved.move(parent=new_parent)
+            self.task.move(parent=new_parent)
 
     # tests added in 3.0
     def test_activity2_retrieve_with_refs(self):
         # setup
-        test_task_ref = 'test-task'
-        test_task_name = 'Test task'
+        test_task_ref = slugify_ref(self.task.name)
         test_task_activity = self.project.activity(ref=test_task_ref)
 
         # testing
         self.assertIsInstance(test_task_activity, Activity2)
-        self.assertTrue(test_task_activity.name, test_task_name)
+        self.assertEqual(self.task, test_task_activity)
 
     def test_activity2_associated_parts(self):
         # setUp
@@ -628,9 +636,7 @@ class TestActivity2SpecificTests(TestBetamax):
         self.assertTrue(len(associated_models) == 3)
 
 
-# @skip('Does not work in PIM2 until KEC-19193 is resolved')
 class TestActivityDownloadAsPDF(TestBetamax):
-
     def test_activity2_download_as_pdf(self):
         # setUp
         activity_name = 'Task - Form'
@@ -648,7 +654,7 @@ class TestActivityDownloadAsPDF(TestBetamax):
     @pytest.mark.skipif("os.getenv('TRAVIS', False) or os.getenv('GITHUB_ACTIONS', False)",
                         reason="Skipping tests when using Travis or Github Actions, as not Auth can be provided")
     def test_activity2_download_as_pdf_async(self):
-        activity_name = 'Task - Form + Tables + Service'
+        activity_name = 'Task - Form'
         activity = self.project.activity(name=activity_name)
 
         # testing
@@ -658,3 +664,58 @@ class TestActivityDownloadAsPDF(TestBetamax):
             pdf_file_called_after_activity = os.path.join(target_dir, activity_name + '.pdf')
             self.assertTrue(pdf_file)
             self.assertTrue(pdf_file_called_after_activity)
+
+    def test_activity2_share_link(self):
+        # setUp
+        test_user = self.client.user(username='testuser')
+
+        activity_name = 'Task - Form'
+        message = 'EXAMPLE_MESSAGE'
+        subject = 'EXAMPLE_SUBJECT'
+        recipient_users = [test_user]
+
+        activity = self.project.activity(name=activity_name)
+
+        activity.share_link(subject=subject,
+                            message=message,
+                            recipient_users=recipient_users)
+
+        # testing
+        notifications = self.client.notifications(subject=subject, message=message,
+                                                  event=NotificationEvent.SHARE_ACTIVITY_LINK)
+        self.assertEqual(self.client.last_response.status_code, requests.codes.ok)
+        self.assertTrue(len(notifications), 1)
+
+        # tearDown
+        notifications[0].delete()
+
+    def test_activity2_share_pdf(self):
+        # setUp
+        test_user = self.client.user(username='testuser')
+
+        activity_name = 'Task - Form'
+        message = 'EXAMPLE_MESSAGE'
+        subject = 'EXAMPLE_SUBJECT'
+        paper_size = PaperSize.A2
+        paper_orientation = PaperOrientation.PORTRAIT
+        recipient_users = [test_user]
+
+        activity = self.project.activity(name=activity_name)
+
+        activity.share_pdf(
+            subject=subject,
+            message=message,
+            recipient_users=recipient_users,
+            paper_size=paper_size,
+            paper_orientation=paper_orientation,
+            include_appendices=False,
+        )
+
+        # testing
+        notifications = self.client.notifications(subject=subject, message=message,
+                                                  event=NotificationEvent.SHARE_ACTIVITY_PDF)
+        self.assertEqual(self.client.last_response.status_code, requests.codes.ok)
+        self.assertTrue(len(notifications), 1)
+
+        # tearDown
+        notifications[0].delete()
